%% init
clear, close all


%% load model
load('data_model');
load('complete_model');

ss_m.A = sysd.A;
ss_m.B = sysd.B;
ss_m.C = sysd.C;
ss_m.D = sysd.D;

ss_m.E = ss_m.B*0;
ss_m.F = ss_m.D*0;

%% MPC setup
N = 10;
weight.Q  = diag([5, 0.5, 5]);
weight.S  = diag([1e-0, 1e-2, 1e-2]);
weight.Sy = diag([1, 1, 1]);
weight.Su = diag([1, 1, 1]);
us = [0.34; 0.69; 0.436];
umin = [0; 0; 0]-us;
umax = [1; 1; 1]-us;
Umin = repmat(umin, N, 1);
Umax = repmat(umax, N, 1);

dumin = [-0.007; -2; -0.05]*Ts;
dumax = [0.007; 0.02; 0.05]*Ts;
dUmin = repmat(dumin, N, 1);
dUmax = repmat(dumax, N, 1);
bdmin = dUmin;
bdmax = dUmax;

mpc_matrix = mpc_matrix_design(ss_m, weight, N);
H = mpc_matrix.H;
MX = mpc_matrix.Mx;
MR = mpc_matrix.Mr;
MU = mpc_matrix.Mu;
F  = mpc_matrix.F;

time = 600;
power_step_time = 100;
run run_simulation
sim('baseline_model_pid', [0, time]);

%% setup simulation
Tmax = time/60; % in "min"
kf   = Tmax*60/Ts;
t    = 0:Ts:kf;
tt   = linspace(0, Tmax*60, kf);

x = zeros(size(ss_m.A, 1), kf);
y = zeros(3, kf);
u = zeros(3, kf);
u_qp = [0, 0, 0]';
nu = 3;
reference.init  = [0; 0; 0];
reference.value = [0; 5; 0];
reference.start = [0; power_step_time/Ts+1; 0];
reference.end   = [kf+N, kf+N, kf+N];
[R, Rplot] = reference_build(N, kf, reference);

%% run simulation
tic;
display('----------------------------')
display('Entering the simulation loop')
fprintf(1,'Status:     ');
for k = 1:1:kf
    % for-loop diagnostic
    for_diag(k, kf)
    
    % MPC controller
    dR = cell2mat(R(k:k+N-1));
    
    % Constraints
    bdmin(1:nu) = dUmin(1:nu) + u_qp;
    bdmax(1:nu) = dUmax(1:nu) + u_qp;
    b           = [bdmax; -bdmin];
    
    g  = MX*x(:, k) + MR*dR + MU*u_qp;
    U = qpas(H, g, F, b, [], [], Umin, Umax);

    u_qp = U(1:nu);
    u_in = u_qp;
    
    x(:, k+1) = ss_m.A*x(:, k) + ss_m.B*u_in;
    y(:, k)   = ss_m.C*x(:, k) + ss_m.D*u_in + y0';
    u(:, k) = u_in;
    
end
fprintf('\n')
time = toc;
cprintf([0.0, 0.0, 0.8], 'Simulation complete\n');
display(['Elapsed time: ' num2str(time) ' s'])
display(['Samples per second: ' num2str(kf/time)])

%% save data
save('mpc_results_simple', 'Rplot', 'y', 'u', 'y_pid', 'u_pid', 'tt', 'y0',...
     'dumin', 'dumax', 'Ts')

%% Display results
close all
display('Displaying results')
figure
subplot(3, 1, 1)
    hold on
    stairs(tt, Rplot(1, :) + y0(1), 'g')
    stairs(tt, y(1, :))
    plot(y_pid(:, 1), y_pid(:, 2), 'm')
    hold off
    ylabel('Pressure')
subplot(3, 1, 2)
    hold on
    stairs(tt, Rplot(2, :) + y0(2), 'g')
    stairs(tt, y(2, :))
    plot(y_pid(:, 1), y_pid(:, 3), 'm')
    hold off
    ylabel('Power')
subplot(3, 1, 3)
    hold on
    stairs(tt, Rplot(3, :) + y0(3), 'g')
    stairs(tt, y(3, :))
    plot(y_pid(:, 1), y_pid(:, 4), 'm')
    hold off
    ylabel('Xw')
fs('Output')

figure
subplot(3, 1, 1)
    hold on
    stairs(tt, u(1, :), 'b')
    plot(y_pid(:, 1), u_pid(:, 2), 'm')
    hold off
    ylabel('Fuel')
subplot(3, 1, 2)
    hold on
    stairs(tt, u(2, :), 'b')
    plot(y_pid(:, 1), u_pid(:, 3), 'm')
    hold off
    ylabel('Steam')
subplot(3, 1, 3)
    hold on
    stairs(tt, u(3, :), 'b')
    plot(y_pid(:, 1), u_pid(:, 4), 'm')
    hold off
    ylabel('Feedwater')
fs('Input')

figure
subplot(3, 1, 1)
    hold on
    stairs(tt(1:end-1), diff(u(1, :))/Ts, 'b')
    stairs(tt(1:end-1), diff(u(1, :))*0+dumin(1)/Ts, 'r')
    stairs(tt(1:end-1), diff(u(1, :))*0+dumax(1)/Ts, 'r')
    hold off
    ylabel('Fuel')
subplot(3, 1, 2)
    hold on
    stairs(tt(1:end-1), diff(u(2, :))/Ts, 'b')
    plot(u_pid(1:end-1, 1), diff(u_pid(:, 3))/Ts, 'm')
%     stairs(tt(1:end-1), diff(u(1, :))*0+dumin(2)/Ts, 'r')
    stairs(tt(1:end-1), diff(u(1, :))*0+dumax(2)/Ts, 'r')
    hold off
    ylabel('Steam')
subplot(3, 1, 3)
    hold on
    stairs(tt(1:end-1), diff(u(3, :))/Ts, 'b')
    stairs(tt(1:end-1), diff(u(1, :))*0+dumin(3)/Ts, 'r')
    stairs(tt(1:end-1), diff(u(1, :))*0+dumax(3)/Ts, 'r')
    hold off
    ylabel('Feedwater')
fs('dInput')
