function mat2tex(M, h, type, align, name)
% Return a LaTeX code of tabular / matrix / array.
%
% INPUT PARAMETERS
%   M       -> matrix
%   h       -> borders {0,1}
%   type    -> 'array', 'tabular', 'matrix'
%   align   -> 'c', 'r', 'l', '.' ('.' is special align by decimal point, 
%              LaTeX command "\newcolumntype{.}{D{.}{.}{-1}}" with package
%              "\usepackage{dcolumn}")
%   name    -> prints along with brackets and matrix name
%
% USAGE
%   M = rand(3,4)
%   mat2tex(M)
%
%         -0.2050 &   1.4090 &  -1.2075 &   0.4889 \\ 
%         -0.1241 &   1.4172 &   0.7172 &   1.0347 \\ 
%          1.4897 &   0.6715 &   1.6302 &   0.7269 \\
%
%   mat2tex(M,1)
% 
%         -0.2050 &   1.4090 &  -1.2075 &   0.4889 \\ \hline 
%         -0.1241 &   1.4172 &   0.7172 &   1.0347 \\ \hline 
%          1.4897 &   0.6715 &   1.6302 &   0.7269 \\ \hline 
%
%   mat2tex(M,1,'tabular')
%
%        \begin{tabular}{|c|c|c|c|}
%         -0.2050 &   1.4090 &  -1.2075 &   0.4889 \\ \hline 
%         -0.1241 &   1.4172 &   0.7172 &   1.0347 \\ \hline 
%          1.4897 &   0.6715 &   1.6302 &   0.7269 \\ \hline 
%        \end{tabular} 
%
%   mat2tex(M, 1, 'array', '.', 'Ax')
%
%       Ax = \left[
%       \begin{array}{|.|.|.|.|}
%        0.5377 &   0.8622 &  -0.4336 &   2.7694 \\ \hline 
%        1.8339 &   0.3188 &   0.3426 &  -1.3499 \\ \hline 
%       -2.2588 &  -1.3077 &   3.5784 &   3.0349 \\ \hline 
%       \end{array} 
%       \right] 
%
% Martin Klauco 08-12-2011, v 3.0

%% Input Check
if (numel(M) < 1)
    error('Matrix M must have at least one element')
end

if (nargin == 1)
    h = 0;
    type = [];   
    align = 'c';
    name = [];
elseif (nargin == 2)
    type = [];
    align = 'c';
    name = [];
    if (ischar(h) == 1)
        error('h can be only 0 or 1. check help mat2tex')
    end
    if (h ~=0 && h ~= 1)
        error('h can be only 0 or 1. check help mat2tex')
    end  
elseif (nargin == 3)
    align = 'c';
    name = [];
    if (h ~=0 && h ~= 1)
        error('h can be only 0 or 1. check help mat2tex')
    end     
    if (strcmp(type,'array') == 0 && strcmp(type,'tabular') == 0 ...
            && strcmp(type,'matrix') == 0)
        error('check help mat2tex for "type" argument values')
    end
elseif (nargin == 4)
    name = [];
end

%% Function Code
if (~isempty(type))
    if strcmp(type, 'array') && (~isempty(name))
        fprintf([name, ' = ' '\\left[']);
    end
    string = char(['\n\\begin{' type '}{']);
    fprintf(string)
    if (h == 0)
        for k=1:1:size(M, 2)
            fprintf(align);
        end
        fprintf('}\n');
        for ROW=1:size(M, 1)
            for COL=1:size(M,2)-1
                fprintf('%8.4f & ', M(ROW,COL));
            end
            fprintf('%8.4f \\\\ \n', M(ROW,end));
        end
    else
        for k=1:1:size(M, 2)
            fprintf(['|' align]);
        end
        fprintf('|}\n');
        for ROW=1:size(M, 1)
            for COL=1:size(M,2)-1
                fprintf('%8.4f & ', M(ROW,COL));
            end
            fprintf('%8.4f \\\\ \\hline \n', M(ROW,end));
        end    
    end
    string = char(['\\end{' type '} \n']);
    fprintf(string)
    if strcmp(type, 'array') && (~isempty(name))
        fprintf('\\right] \n');
    end
else
    fprintf('\n');
    if (h == 0)
    for ROW=1:size(M, 1)
        for COL=1:size(M,2)-1
            fprintf('%8.4f & ', M(ROW,COL));
        end
        fprintf('%8.4f \\\\ \n', M(ROW,end));
    end
    else
        for ROW=1:size(M, 1)
            for COL=1:size(M,2)-1
                 fprintf('%8.4f & ', M(ROW,COL));
            end
        fprintf('%8.4f \\\\ \\hline \n', M(ROW,end));
        end
    end
    fprintf('\n');
end
end
