function [mpc_matrix, index] = mpc_matrix_design(system, weight, N)
%% Function for general MPC matrix design
%
%   based on JBJ Lectures 2011, DTU
%

%% source code
A = system.A;
B = system.B;
C = system.C;
D = system.D;
E = system.E;
F = system.F;
nu = size(B, 2);
ny = size(C, 1);

%% Prediction and Contraints matrices
H_GAMMA_ZERO = C*A*B.*0;
H_GAMMAD_ZERO = C*A*E.*0;
% Qz_ZERO = weight.Q.*0;
S_ZERO = weight.S.*0;
B_I = eye(nu, nu);
C_I = eye(ny, ny);

H_GAMMA = cell(N, 1);
H_GAMMAD = cell(N, 1);
H_GAMMAY = cell(N, 1);

for k = 1:1:N
    H_GAMMA(k, 1) = {C*(A^(k-1))*B};
    H_GAMMAD(k, 1) = {C*(A^(k-1))*E};
    if k == 1
        H_GAMMAY(k, 1) = {C*(A^(k-1))*B-D};
    else
        H_GAMMAY(k, 1) = {C*(A^(k-1))*B - C*(A^(k-2))*B};
    end
end

index = zeros(N, N);

GAMMA  = cell(N, N);
GAMMAD = cell(N, N);
GAMMAY = cell(N, N);

Mu = cell(N, 1);
PHI = cell(N, 1);
PHIY = cell(N, 1);
for j = 1:1:N
    PHI(j, 1) = {C*A^(j-1)};
    if j == 1
        PHIY(j, 1) = {C*A^(j-1)};
    else
        PHIY(j, 1) = {C*A^(j-1) - C*A^(j-2)};
    end
    Mu(j, 1) = {B_I.*0};
    for k = 1:1:N
        idx = j - k;
        
        % Fill Output Prediction GAMMA, GAMMAD matrix
        if idx > 0
            GAMMA(j, k)  = H_GAMMA(idx);
            GAMMAD(j, k) = H_GAMMAD(idx);
            GAMMAY(j, k) = H_GAMMAY(idx);
        else
            GAMMA(j, k)  = {H_GAMMA_ZERO};
            GAMMAY(j, k) = {H_GAMMA_ZERO};
            GAMMAD(j, k) = {H_GAMMAD_ZERO};
        end
       if idx == 0
           GAMMA(j, k)  = {D};
           GAMMAY(j, k) = {D};
           GAMMAD(j, k) = {F};
       end

        % this matrix is just for checking if the indexes are asigned
        % correctly
        index(j, k) = j - k;
    end
end
Mu(1, 1) = {weight.S};
LAMBDA = cell(N, N);
for j = 1:1:N
    for k = 1:1:N
        idx = j - k;
        if idx == 0
            LAMBDA(j, k) = {B_I};
        elseif idx == 1
            LAMBDA(j, k) = {-B_I};
        else
            LAMBDA(j, k) = {0.*B_I};
        end
    end
end

LAMBDA_Y = cell(N, N);
for j = 1:1:N
    for k = 1:1:N
        idx = j - k;
        if idx == 0
            LAMBDA_Y(j, k) = {C_I};
        elseif idx == 1
            LAMBDA_Y(j, k) = {-C_I};
        else
            LAMBDA_Y(j, k) = {0.*C_I};
        end
    end
end

Hs = cell(N, N);
for j = 1:1:N
    for k = 1:1:N
        Hs(j, k) = {weight.S.*0};
        if index(j, k) == 0
            Hs(j, k) = {2.*weight.S};
        end
        if index(j, k) == -1 || index(j, k) == 1
            Hs(j, k) = {-weight.S};
        end
    end
end
Hs(end, end) = {weight.S};

% Correct the first positon of Mu matrix
Mu(1, 1) = {-weight.S};

% Correct input-rates movement
LAMBDA   = cell2mat(LAMBDA);
LAMBDA_Y = cell2mat(LAMBDA_Y);

%% convert into matrices
GAMMA      = cell2mat(GAMMA);
GAMMAD     = cell2mat(GAMMAD);
GAMMAY     = cell2mat(GAMMAY);
PHI        = cell2mat(PHI);
PHIY       = cell2mat(PHIY);
Hs         = cell2mat(Hs);
Qz         = kron(eye(N), weight.Q);
% if size(LAMBDA, 1) < size(GAMMA, 1)
%     sl = size(GAMMA, 1) - size(LAMBDA, 1)
%     LAMBDA     = padarray(LAMBDA, [sl 0], 'post');
% if size(LAMBDA, 2) < size(GAMMA, 2)
%     sl = size(GAMMA, 2) - size(LAMBDA, 2);
%     LAMBDA     = padarray(LAMBDA, [0 sl], 'post');
% end
%% Outputs
mpc_matrix.GAMMA      = GAMMA;
mpc_matrix.GAMMAD     = GAMMAD;
mpc_matrix.GAMMAY     = GAMMAY;
mpc_matrix.PHI        = PHI;
mpc_matrix.PHIY       = PHIY;
mpc_matrix.Qz         = Qz;
mpc_matrix.Mu         = cell2mat(Mu);
mpc_matrix.LAMBDA     = LAMBDA;
mpc_matrix.LAMBDA_Y   = LAMBDA_Y;

mpc_matrix.Hs = Hs;
mpc_matrix.Mx = GAMMA'*Qz*PHI;
mpc_matrix.Mr = -GAMMA'*Qz;
mpc_matrix.Md = GAMMA'*Qz*GAMMAD;

H = GAMMA'*Qz*GAMMA + Hs;
H   = 0.5.*(H + H');
H1  = blkdiag(H, kron(eye(N), weight.Su));
H2  = blkdiag(H1, kron(eye(N), weight.Sy));
mpc_matrix.H = H;
mpc_matrix.H1 = H1;
mpc_matrix.H2 = H2;

%% Soft on delta u and y
sL = size(LAMBDA, 1);
sG = size(GAMMA, 1);
IL = eye(sL, sL);
IG = eye(sG, sL);
ZL = zeros(sL, sL);
ZG = zeros(sG, sL);

F = [ LAMBDA;
     -LAMBDA];

F1 = [ LAMBDA, -IL;
      -LAMBDA, -IL];

F2 = [ LAMBDA, -IL, ZL;
      -LAMBDA, -IL, ZL;
       GAMMA,   ZG,-IG;
      -GAMMA,   ZG, ZG;];

F3 = [ GAMMAY;
      -GAMMAY];

F4 = [ GAMMAY;
      -GAMMAY;
       GAMMA;
      -GAMMA;];
  
mpc_matrix.F = F;
mpc_matrix.F1 = F1;
mpc_matrix.F2 = F2;
mpc_matrix.F3 = F3;
mpc_matrix.F4 = F4;
% mpc_matrix.F5 = F5;
end